{% include 'header.jinja2' %}

## Document Store

This is a document store component. It is used to store documents and native files.

This store is defined to be used for:

{% if component.store_purpose == 'TRAINING' %}
🧐Training documents that are labeled and used to train models
{% endif %}
{% if component.store_purpose == 'OPERATIONAL' %}
📚Operational documents that are processed to label and extract data
{% endif %}

### Options

{% if component.searchable %}
✅ Searchable
{% endif %}

{% if component.show_thumbnails %}
✅ Show thumbnails
{% endif %}

{% if component.show_search %}
✅ Show search in UI
{% endif %}

{% if component.delete_protection %}
✅ Delete protection
{% endif %}