{% include 'header.jinja2' %}

## Data Store

This is a data store component, which means it will hold structured data in the form of
data objects and data attributes.

### Options

{% if component.allow_data_editing %}
✅ Allow data editing
{% endif %}

{% if component.delete_protection %}
✅ Delete protection
{% endif %}

